<?php
/*
Plugin Name: Embed Link
Plugin URI: https://wordpress.org/plugins/embed-link/
Description: Embed any link in Wordpress using it's open graph thumbnail image
Version: 1.0.0
Author: James Low
Author URI: http://jameslow.com
License: MIT License
*/

class EmbedLink {
	public static $PREFIX = 'embedlink/v1';
	public static $METHOD = 'oembed';

	private static $_version = null;
	public static function version() {
		if (!self::$_version) {
			if (!function_exists('get_plugin_data')){
				require_once(ABSPATH . 'wp-admin/includes/plugin.php');
			}
			$data = get_plugin_data(__FILE__);
			self::$_version = $data['Version'];
		}
		return self::$_version;
	}

	public static function add_hooks() {
		//https://developer.wordpress.org/reference/classes/wp_oembed/
		add_action('init', array('EmbedLink', 'init'), 99);
		add_action('wp_enqueue_scripts', array('EmbedLink', 'wp_enqueue_scripts'));
		//add_action('wp_head', array('EmbedLink', 'wp_head'));
		//add_filter('oembed_providers', array('EmbedLink', 'oembed_providers'), 99); //Last so that other providers take priority
		//add_filter('oembed_fetch_url', array('EmbedLink', 'oembed_fetch_url'));
		add_action( 'rest_api_init', function() {
			register_rest_route(self::$PREFIX, self::$METHOD, array(
			  'methods' => 'GET',
			  'callback' => array('EmbedLink', 'oembed'),
			));
		});
	}
	public static function init() {
		wp_oembed_add_provider('#https?://.*#i', self::provider_url(), true);
	}
	public static function wp_enqueue_scripts() {
		//wp_enqueue_style('embed-link', plugin_dir_url(__FILE__).'css/style.css', array(), EmbedLink::version());
	}
	public static function provider_url() {
		return get_rest_url().self::$PREFIX.'/'.self::$METHOD;
	}
	//public static function oembed_providers($providers) {
	//	$providers[] = ['#https?://.*' => [self::provider_url(), true]];
	//	return $providers;
	//}
	//public static function oembed_fetch_url($provider, $url, $args) {
	//
	//}
	public static function oembed() {
		//https://oembed.com
		$maxwidth = (int) $_GET['maxwidth'];
		$maxheight = (int) $_GET['maxheight'];
		$geturl = $_GET['url'];
		$url = esc_url($geturl);
		$author_name = '';
		$title = '';
		$html = '';
		if (filter_var($geturl, FILTER_VALIDATE_URL)) {
			$http = new EmbedLinkHttp($geturl);
			$data = $http->Get(array(), true);
			$contenttype = $data['header']['content-type'];
			if (strpos($contenttype, 'text/html') !== false) {
				//<meta property="og:image" content="https://i.insider.com/560e7519eab8eac97da5a044?width=1200&amp;format=jpeg">
				$dom = new DomDocument();
				@$dom->loadHTML($data['body']);
				$title = $dom->getElementsByTagName('title')->item(0)->textContent;
				$meta = $dom->getElementsByTagName('meta');
				$thumbnail = '';
				foreach ($meta as $node) {
					$type = $node->getAttribute('property');
					if ($type == 'og:image') {
						$thumbnail = $node->getAttribute('content');
						break;
					}
				}
				$html = '<a href="'.$url.'" target="_blank"><img style="width:100%;" class="aligncenter wp_embed_link_image" src="'.$thumbnail.'" /></a>';
				//TODO: can we return a caption?
				//<a href="'.$url.'" target="_blank"><div class="wp_embed_link_title">'.$title.'</div></a>
			} elseif (strpos($contenttype, 'image/') !== false) {
				$html = '<a href="'.$url.'" target="_blank"><img style="width:100%;" class="aligncenter wp_embed_link_image" src="'.$thumbnail.'" /></a>';
			//Wordpress supports Audio/Video already
			//} elseif (strpos($contenttype, 'video/') !== false) {
			//	$html = '<audio controls><source src="'.$url.'" type="'.$contenttype.'" /></audio>';
			//} elseif (strpos($contenttype, 'audio/') !== false) {
			//	$html = '<video controls><source src="'.$url.'" type="'.$contenttype.'" /></audio>';
			} else {
				http_response_code(501);
				exit;
			}
		} else {
			http_response_code(501);
			exit;
		}

		$result = array(
			'version' => '1.0',
			'type' => 'rich',
			'provider_name' => get_bloginfo('name'),
			'provider_url' => self::provider_url(),
			'width' => '100%',
			'height' => 'auto',
			'title' => $title,
			'author_name' => $author_name ? $author_name : $title, //<meta property="author" content="James Rickards">
			'author_url' => $geturl,
			'html' => '<div class="wp_embed_link">'.$html.'</div>'
		);
		return $result;
	}
}
EmbedLink::add_hooks();


class EmbedLinkHttp {
	const GET = 'GET';
	const PUT = 'PUT';
	const POST = 'POST';
	const DELETE = 'DELETE';
	const URL_ENCODED = 'application/x-www-form-urlencoded';
	const HTTP = 'http';
	const HEADER = 'header';
	const BODY = 'body';

	var $fp;			// the socket
	var $url;			// full URL
	var $host;			// HTTP host
	var $protocol;		// protocol (HTTP/HTTPS)
	var $uri;			// request URI
	var $port;			// port
	var $timeout;		// timeout
	var $keepalive;		// keep tcp alive

	function __construct($url, $keepalive = false, $timeout = 1000) {
		$this->SetUrl($url);
		$this->timeout = $timeout;
		$this->keepalive = $keepalive;
	}

	function SetUrl($url) {
		$this->Close();
		$this->url = $url;
		$this->scanUrl();
	}

	function Close() {
		if ($this->fp && is_resource($this->fp)) {
			fclose($this->fp);
			$this->fp = null;
		}
	}

	//Scan URL and prepopulate variables
	function scanUrl() {
		$req = $this->url;
	
		$pos = strpos($req, '://');
		$this->protocol = strtolower(substr($req, 0, $pos));
	
		$req = substr($req, $pos+3);
		$pos = strpos($req, '/');
		if($pos === false) {
			$pos = strlen($req);
		}
		$host = substr($req, 0, $pos);
	
		if(strpos($host, ':') !== false) {
			list($this->host, $this->port) = explode(':', $host);
		} else {
			$this->host = $host;
			$this->port = ($this->protocol == 'https') ? 443 : 80;
		}
	
		$this->uri = substr($req, $pos);
		if($this->uri == '') {
			$this->uri = '/';
		}
	}

	function SetUri($uri) {
		$this->uri = $uri;
	}

	//Encode a value in the header
	function HeaderEncode($header, $value) {
		$crlf = "\r\n";
		return $header.': '.$value.$crlf;
	}

	//Basic header common to both GET and POST
	function BasicHeader($method, $params, $headerarray) {
		$crlf = "\r\n";
		if (!$headerarray['Host']) {
			$headerarray['Host'] = $this->host;
		}
		$headerarray['Connection'] = ($this->keepalive?'keep-alive':'close');
		$result = $method.' '.$this->uri.($params==''?'':'?'.$params).' HTTP/1.1'.$crlf;
		foreach ($headerarray as $header => $value) {
			$result .= $this->HeaderEncode($header,$value);
		}
		$result .= $crlf;
		return $result;
	}

	//Encode request param
	function ParamEncode($key, $value) {
		return urlencode($key).'='.urlencode($value);
	}

	//Encode request params
	function RequestEncode($paramarray) {
		$content = '';
		//Encode all post parameters
		if ($paramarray) {
			foreach($paramarray as $key => $value) {
				if ($content == '') {
					$content = $this->ParamEncode($key,$value);
				} else {
					$content .= '&'.$this->ParamEncode($key,$value);
				}
			}
		}
		return $content;
	}

	//Content Lengt
	function ContentLength($content) {
		return strlen($content);
	}

	//Get request
	function Get($paramarray = array(), $details = false, $headerarray = array()) {
		return $this->Body(self::GET,$paramarray,$details,$headerarray);
	}

	//Post request
	function Post($paramarray = array(), $details = false, $headerarray = array()) {
		return $this->Body(self::POST,$paramarray,$details,$headerarray);
	}

	//Put request
	function Put($paramarray = array(), $details = false, $headerarray = array()) {
		return $this->Body(self::PUT,$paramarray,$details,$headerarray);
	}

	//Delete request
	function Delete($paramarray = array(), $details = false, $headerarray = array()) {
		return $this->Body(self::DELETE,$paramarray,$details,$headerarray);
	}

	function Query($method, $paramarray = array(), $details = false, $headerarray = array()) {
		$req = $this->BasicHeader($method, $this->RequestEncode($paramarray), $headerarray);
		return $this->Request($req,$details);
	}

	function Body($method, $paramarray = array(), $details = false, $headerarray = array()) {
		if (is_array($paramarray)) {
			$content = $this->RequestEncode($paramarray);
			if (!$headerarray['Content-Type']) {
				$headerarray['Content-Type'] = self::URL_ENCODED;
			}
		} else {
			$content = $paramarray;
		}
		if (!$headerarray['Content-Length']) {
			$headerarray['Content-Length'] = $this->ContentLength($content);
		}
		$req = $this->BasicHeader($method,'',$headerarray).$content;
		return $this->Request($req,$details);
	}

	//General request
	function Request($req) {
		$crlf = "\r\n";
		//Fetch the data
		if (!$this->keepalive || !$this->fp || !is_resource($this->fp)) {
			$this->fp = fsockopen(($this->protocol == 'https'?'ssl://':'').$this->host, $this->port);
			stream_set_timeout($this->fp, 0, $this->timeout * 1000);
		}
		fwrite($this->fp, $req);
		$response = '';
		$line = '';
		$lastbyte = null;
		$contentlength = 0;
		$contentstart = false;
		$contenttype = '';
		while($this->fp && is_resource($this->fp) && !feof($this->fp)) {
			if (!$contentstart) {
				$byte = fread($this->fp, 1);
				$response .= $byte;
				$line .= $byte;
				if ($lastbyte == "\r" && $byte == "\n") {
					if ($line == $crlf) {
						$contentstart = true;
					} elseif (strpos(strtolower($line),'content-length') !== false) {
						$contentlength = trim(substr($line, strpos($line,':')+1));
					} elseif (strpos(strtolower($line),'content-type') !== false) {
						$contenttype = trim(substr($line, strpos($line,':')+1));
					}
					$line = '';
				}
				$lastbyte = $byte;
			} else {
				//Only read body for html
				//$contentlength > 0 && 
				if (strpos(strtolower($contenttype), 'text/html') !== false) {
					while($this->fp && is_resource($this->fp) && !feof($this->fp)) {
						$response .= fread($this->fp, 1024);
					}
					//$response .= fread($this->fp, $contentlength);
				} else {
					$response .= $crlf; //Add extra carriage return for split below because we only add one before
				}
				break;
			}
		}
		$this->Close();
	
		//Split header and body
		$pos = strpos($response, $crlf . $crlf);
		if($pos === false) {
			return($response);
		}
		$header = substr($response, 0, $pos);
		$body = substr($response, $pos + 2 * strlen($crlf));
	
		//Parse headers
		$headers = array();
		$lines = explode($crlf, $header);
		$firsttime = true;
		foreach($lines as $line) {
			if ($firsttime) {
				$codes = explode(" ", $line);
				$code['version'] = $codes[0];
				$code['code'] = intval($codes[1]);
				$code['message'] = $codes[2];
				$firsttime = false;
			}
			if(($pos = strpos($line, ':')) !== false) {
				$headers[strtolower(trim(substr($line, 0, $pos)))] = trim(substr($line, $pos+1));
			}
		}
	
		//Redirection
		if(isset($headers['location'])) {
			//TODO: 1) Redirects currently only work for GET requests
			//TODO: 2) Eventually handle keep alive here too
			$http = new EmbedLinkHttp($headers['location']);
			$req = $http->BasicHeader(self::GET, $this->RequestEncode($paramarray), $headerarray);
			return $http->Request($req, $details);
		} else {
			if (strtolower($headers['transfer-encoding']) == 'chunked') {
				$body = $this->unchunkHttp11($body);
			}
			$result[self::HTTP] = $code;
			$result[self::HEADER] = $headers;
			$result[self::BODY] = $body;
			return $result;
		}
	
	}

	function unchunkHttp11($data) {
		$fp = 0;
		$outData = "";
		while ($fp < strlen($data)) {
			$rawnum = substr($data, $fp, strpos(substr($data, $fp), "\r\n") + 2);
			$num = hexdec(trim($rawnum));
			$fp += strlen($rawnum);
			$chunk = substr($data, $fp, $num);
			$outData .= $chunk;
			$fp += strlen($chunk);
		}
		return $outData;
	}

	//Download URL to string, included for backwards compatibilty with versions that did have seperate GET/POST
	function DownloadToString() {
		return $this->Get(null,false);
	}
}